/*
	Created by:		Esposito Vittorio
	GitHub:			https://github.com/VittorioEsposito	
	
	J1850-PWM-Encoding-Library
	
	Arduino library which allow to communicate on J1850-PWM mode. 
	It works with both wire and radio transmissions. 
	You just have to choose your bit-rate (lower is better). 
	It doesn't have either a header nor a parity check. 
	These functions could be implemented at a higher protocol layer. 
	This library has been tested also with FSK module RTX-mid-868.
	
*/

#include <Arduino.h>
#include <PwmEncoder.h>


PwmEncoder::PwmEncoder(uint8_t TX_PIN)
{
	_TX_PIN = TX_PIN;
	pinMode(_TX_PIN, OUTPUT);
}

PwmEncoder::PwmEncoder(uint8_t TX_PIN, uint8_t MODE_PIN, uint8_t EN_PIN)
{
	_TX_PIN = TX_PIN;
	pinMode(_TX_PIN, OUTPUT);
	_MODE_PIN = MODE_PIN;
	pinMode(_MODE_PIN, OUTPUT);
	_EN_PIN = EN_PIN;
	pinMode(EN_PIN, OUTPUT);
}

void PwmEncoder::setTxPin(uint8_t TX_PIN)
{
	_TX_PIN = TX_PIN;
	pinMode(_TX_PIN, OUTPUT);
}

void PwmEncoder::setEnablePin(uint8_t EN_PIN)
{
	_EN_PIN = EN_PIN;
	pinMode(EN_PIN, OUTPUT);
}

void PwmEncoder::setTxRxModePin(uint8_t MODE_PIN)
{
	_MODE_PIN = MODE_PIN;
	pinMode(_MODE_PIN, OUTPUT);
}

uint8_t PwmEncoder::getTxPin()
{
	return _TX_PIN;
}

void PwmEncoder::begin(uint32_t BITRATE)
{
	_BITRATE  = BITRATE;
	BIT_TIME  = 1000000 / BITRATE;  // uS
	ONE_THIRD = BIT_TIME / 3;
	TWO_THIRD = BIT_TIME * 2 / 3;
}

void PwmEncoder::powerDown()
{
	digitalWrite(_EN_PIN, LOW);
	delay(15);
}

void PwmEncoder::powerUp()
{
	digitalWrite(_EN_PIN, HIGH);
	delay(15);
}

void PwmEncoder::println(String str)
{
	str += '\n';
	str += '\r';
	print(str);
}

void PwmEncoder::print(String str)
{
	uint16_t size = str.length() + 1;
	char txt[size];
	str.toCharArray(txt, size);
	uint16_t i;
	for (i = 0; i < size; i++) {
		PwmEncoder::print(txt[i]);
	}
}

void PwmEncoder::print(char txt[])
{
	uint16_t size = strlen(txt);
	uint16_t i;
	for (i = 0; i < size; i++) {
		PwmEncoder::print(txt[i]);
	}
}

void PwmEncoder::print(char c)
{
	int i;
	for (i = 7; i >= 0; i--) {
		PwmEncoder::send((c >> i) % 2);
	}
}

void PwmEncoder::print(uint8_t n) {
	int i;
	for (i = 7; i >= 0; i--) {
		PwmEncoder::send((n >> i) % 2);
	}
}

void PwmEncoder::send(bool b)
{
	if (!b) {
		/* Send 33% HIGH, 66% LOW  */
		digitalWrite(_TX_PIN, HIGH);
		delayMicroseconds(ONE_THIRD);
		digitalWrite(_TX_PIN, LOW);
		delayMicroseconds(TWO_THIRD);
	} else {
		/* Send 66% HIGH, 33% LOW  */
		digitalWrite(_TX_PIN, HIGH);
		delayMicroseconds(TWO_THIRD);
		digitalWrite(_TX_PIN, LOW);
		delayMicroseconds(ONE_THIRD);
	}
}

void PwmEncoder::enableTxMode()
{
	digitalWrite(_MODE_PIN, HIGH);
	delayMicroseconds(500);
}

void PwmEncoder::enableRxMode()
{
	digitalWrite(_MODE_PIN, LOW);
	delayMicroseconds(500);
}
